package javax.swing;

import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleContext;
import java.util.ArrayList;
import java.awt.Rectangle;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Container;
import sun.awt.SunToolkit;
import java.awt.Component;
import java.util.Hashtable;
import javax.accessibility.Accessible;

public class JLayeredPane extends JComponent implements Accessible
{
    public static final Integer DEFAULT_LAYER;
    public static final Integer PALETTE_LAYER;
    public static final Integer MODAL_LAYER;
    public static final Integer POPUP_LAYER;
    public static final Integer DRAG_LAYER;
    public static final Integer FRAME_CONTENT_LAYER;
    public static final String LAYER_PROPERTY = "layeredContainerLayer";
    private Hashtable<Component, Integer> componentToLayer;
    private boolean optimizedDrawingPossible;
    
    public JLayeredPane() {
        this.optimizedDrawingPossible = true;
        this.setLayout(null);
    }
    
    private void validateOptimizedDrawing() {
        boolean n = false;
        synchronized (this.getTreeLock()) {
            Component[] components = this.getComponents();
            int length = components.length;
            int i = 0;
            while (i < length) {
                Component obj = components[i];
                Integer n2 = null;
                if ((SunToolkit.isInstanceOf(obj, "javax.swing.JInternalFrame") || (obj instanceof JComponent && (n2 = (Integer)((JComponent)obj).getClientProperty("layeredContainerLayer")) != null)) && (n2 == null || !n2.equals(JLayeredPane.FRAME_CONTENT_LAYER))) {
                    n = true;
                    break;
                }
                ++i;
            }
        }
        if (!(n == 0)) {
            this.optimizedDrawingPossible = false;
        }
        else {
            this.optimizedDrawingPossible = true;
        }
    }
    
    @Override
    protected void addImpl(Component comp, Object constraints, int n) {
        int n2;
        if ((constraints instanceof Integer)) {
            n2 = (int)constraints;
            this.setLayer(comp, n2);
        }
        else {
            n2 = this.getLayer(comp);
        }
        super.addImpl(comp, constraints, this.insertIndexForLayer(n2, n));
        comp.validate();
        comp.repaint();
        this.validateOptimizedDrawing();
    }
    
    @Override
    public void remove(int n) {
        Component component = this.getComponent(n);
        super.remove(n);
        if (component != null && !(component instanceof JComponent)) {
            this.getComponentToLayer().remove(component);
        }
        this.validateOptimizedDrawing();
    }
    
    @Override
    public void removeAll() {
        Component[] components = this.getComponents();
        Hashtable<Component, Integer> componentToLayer = this.getComponentToLayer();
        for (int i = components.length - 1; i >= 0; --i) {
            Component key = components[i];
            if (key != null && !(key instanceof JComponent)) {
                componentToLayer.remove(key);
            }
        }
        super.removeAll();
    }
    
    @Override
    public boolean isOptimizedDrawingEnabled() {
        return this.optimizedDrawingPossible;
    }
    
    public static void putLayer(JComponent component, int value) {
        component.putClientProperty("layeredContainerLayer", new Integer(value));
    }
    
    public static int getLayer(JComponent component) {
        Integer n;
        if ((n = (Integer)component.getClientProperty("layeredContainerLayer")) != null) {
            return (int)n;
        }
        return (int)JLayeredPane.DEFAULT_LAYER;
    }
    
    public static JLayeredPane getLayeredPaneAbove(Component component) {
        if (component == null) {
            return null;
        }
        Container container;
        for (container = component.getParent(); container != null && !(container instanceof JLayeredPane); container = container.getParent()) {}
        return (JLayeredPane)container;
    }
    
    public void setLayer(Component component, int n) {
        this.setLayer(component, n, -1);
    }
    
    public void setLayer(Component component, int n, int n2) {
        Integer objectForLayer = this.getObjectForLayer(n);
        if (n == this.getLayer(component) && n2 == this.getPosition(component)) {
            this.repaint(component.getBounds());
            return;
        }
        if ((component instanceof JComponent)) {
            ((JComponent)component).putClientProperty("layeredContainerLayer", objectForLayer);
        }
        else {
            this.getComponentToLayer().put(component, objectForLayer);
        }
        if (component.getParent() == null || component.getParent() != this) {
            this.repaint(component.getBounds());
            return;
        }
        this.setComponentZOrder(component, this.insertIndexForLayer(component, n, n2));
        this.repaint(component.getBounds());
        return;
    }
    
    public int getLayer(Component key) {
        Integer n;
        if ((key instanceof JComponent)) {
            n = (Integer)((JComponent)key).getClientProperty("layeredContainerLayer");
        }
        else {
            n = (Integer)this.getComponentToLayer().get(key);
        }
        if (n == null) {
            return (int)JLayeredPane.DEFAULT_LAYER;
        }
        return (int)n;
    }
    
    public int getIndexOf(Component component) {
        int componentCount = this.getComponentCount();
        int i = 0;
        while (i < componentCount) {
            if (component == this.getComponent(i)) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    public void moveToFront(Component component) {
        this.setPosition(component, 0);
    }
    
    public void moveToBack(Component component) {
        this.setPosition(component, -1);
    }
    
    public void setPosition(Component component, int n) {
        this.setLayer(component, this.getLayer(component), n);
    }
    
    public int getPosition(Component component) {
        int n = 0;
        this.getComponentCount();
        int index = this.getIndexOf(component);
        if (index == -1) {
            return -1;
        }
        int layer = this.getLayer(component);
        int i = index - 1;
        while (i >= 0) {
            if (this.getLayer(this.getComponent(i)) == layer) {
                ++n;
                --i;
            }
            else {
                return n;
            }
        }
        return n;
    }
    
    public int highestLayer() {
        if (this.getComponentCount() > 0) {
            return this.getLayer(this.getComponent(0));
        }
        return 0;
    }
    
    public int lowestLayer() {
        int componentCount = this.getComponentCount();
        if (componentCount > 0) {
            return this.getLayer(this.getComponent(componentCount - 1));
        }
        return 0;
    }
    
    public int getComponentCountInLayer(int n) {
        int n2 = 0;
        for (int componentCount = this.getComponentCount(), i = 0; i < componentCount; ++i) {
            int layer = this.getLayer(this.getComponent(i));
            if (layer == n) {
                ++n2;
            }
            else if (n2 <= 0) {
                if (layer < n) {
                    break;
                }
            }
            else {
                break;
            }
        }
        return n2;
    }
    
    public Component[] getComponentsInLayer(int n) {
        int n2 = 0;
        Component[] array = new Component[this.getComponentCountInLayer(n)];
        for (int componentCount = this.getComponentCount(), i = 0; i < componentCount; ++i) {
            int layer = this.getLayer(this.getComponent(i));
            if (layer == n) {
                array[n2++] = this.getComponent(i);
            }
            else if (n2 <= 0) {
                if (layer < n) {
                    break;
                }
            }
            else {
                break;
            }
        }
        return array;
    }
    
    @Override
    public void paint(Graphics graphics) {
        if (this.isOpaque()) {
            Rectangle clipBounds = graphics.getClipBounds();
            Color color = this.getBackground();
            if (color == null) {
                color = Color.lightGray;
            }
            graphics.setColor(color);
            if (clipBounds != null) {
                graphics.fillRect(clipBounds.x, clipBounds.y, clipBounds.width, clipBounds.height);
            }
            else {
                graphics.fillRect(0, 0, this.getWidth(), this.getHeight());
            }
        }
        super.paint(graphics);
    }
    
    protected Hashtable<Component, Integer> getComponentToLayer() {
        if (this.componentToLayer == null) {
            this.componentToLayer = new Hashtable(4);
        }
        return this.componentToLayer;
    }
    
    protected Integer getObjectForLayer(int value) {
        Integer n;
        switch (value) {
            case 0: {
                n = JLayeredPane.DEFAULT_LAYER;
                break;
            }
            case 100: {
                n = JLayeredPane.PALETTE_LAYER;
                break;
            }
            case 200: {
                n = JLayeredPane.MODAL_LAYER;
                break;
            }
            case 300: {
                n = JLayeredPane.POPUP_LAYER;
                break;
            }
            case 400: {
                n = JLayeredPane.DRAG_LAYER;
                break;
            }
            default: {
                n = new Integer(value);
                break;
            }
        }
        return n;
    }
    
    protected int insertIndexForLayer(int n, int n2) {
        return this.insertIndexForLayer(null, n, n2);
    }
    
    private int insertIndexForLayer(Component component, int n, int n2) {
        int n3 = -1;
        int n4 = -1;
        int componentCount = this.getComponentCount();
        ArrayList<Component> list = new ArrayList(componentCount);
        for (int i = 0; i < componentCount; ++i) {
            if (this.getComponent(i) != component) {
                list.add(this.getComponent(i));
            }
        }
        int size = list.size();
        int j = 0;
        while (j < size) {
            int layer = this.getLayer((Component)list.get(j));
            if (n3 == -1 && layer == n) {
                n3 = j;
            }
            if (layer < n) {
                if (j == 0) {
                    n3 = 0;
                    n4 = 0;
                    break;
                }
                n4 = j;
                break;
            }
            else {
                ++j;
            }
        }
        if (n3 == -1 && n4 == -1) {
            return size;
        }
        if (n3 != -1 && n4 == -1) {
            n4 = size;
        }
        if (n4 != -1 && n3 == -1) {
            n3 = n4;
        }
        if (n2 == -1) {
            return n4;
        }
        if (n2 > -1 && n3 + n2 <= n4) {
            return n3 + n2;
        }
        return n4;
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",optimizedDrawingPossible=" + (this.optimizedDrawingPossible ? "true" : "false");
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JLayeredPane.AccessibleJLayeredPane();
        }
        return this.accessibleContext;
    }
    
    static {
        DEFAULT_LAYER = new Integer(0);
        PALETTE_LAYER = new Integer(100);
        MODAL_LAYER = new Integer(200);
        POPUP_LAYER = new Integer(300);
        DRAG_LAYER = new Integer(400);
        FRAME_CONTENT_LAYER = new Integer(-30000);
    }
    
    protected class AccessibleJLayeredPane extends JComponent.AccessibleJComponent
    {
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.LAYERED_PANE;
        }
    }
}
